//
//     Copyright (C) 2003 - 2007 SophiaCradle, Inc.
//     All rights reserved.
//
//     File: Overlap.cpp
//

#include "Overlap.hpp"

// Macro used to declare the application types
SFMTYPEDEFCLASS(MyRect)

// Class for collision detection
class MyRect {

private:

    SFXRectangle _rect;

public:

    MyRect(SFXRectangleConstRef rect) : _rect(rect) {
    }
    MyRect(SFXCircleConstRef circle);
    UInt16 GetLine(MyRectConstRef parent);
};

MyRect::MyRect(SFXCircleConstRef circle)
{
    SFXPixel origin(circle.GetOrigin());
    SInt16 radius = circle.GetRadius();
    SFXPixel start(origin);
    start.SubX(radius);
    start.SubY(radius);
    SFXPixel end(origin);
    end.AddX(radius);
    end.AddY(radius);
    _rect.Set(SFXRectangle(start, end));
}

//
// Get the object position relative to that of parent.
// LINE_NONE   : superpose none
// LINE_LEFT   : Left
// LINE_BOTTOM : Botton
// LINE_RIGHT  : Right
// LINE_TOP    : Top
// LINE_INSIDE : Inside of parent
//
UInt16 MyRect::GetLine(MyRectConstRef parent)
{
    if (!_rect.IntersectsWith(parent._rect)) {
        return LINE_NONE;
    }

    UInt16 ret = 0;

    SFXRectangle intersection(_rect.Intersection(parent._rect));
    if (intersection.GetLeft() == parent._rect.GetLeft()) {
        ret |= LINE_LEFT;
    }
    if (intersection.GetBottom() == parent._rect.GetBottom()) {
        ret |= LINE_BOTTOM;
    }
    if (intersection.GetRight() == parent._rect.GetRight()) {
        ret |= LINE_RIGHT;
    }
    if (intersection.GetTop() == parent._rect.GetTop()) {
        ret |= LINE_TOP;
    }

    if (!ret) {
        ret = LINE_INSIDE;
    }

    return ret;
}

UInt16 operator*(SFXCircleConstRef child, SFXRectangleConstRef parent)
{
    return MyRect(child).GetLine(MyRect(parent));
}

UInt16 operator*(SFXRectangleConstRef child, SFXRectangleConstRef parent)
{
    return MyRect(child).GetLine(MyRect(parent));
}
